/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.client.mock.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.FixMethodOrder;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runners.MethodSorters;
import org.ow2.petals.registry.client.api.exception.ConnectionErrorException;
import org.ow2.petals.registry_overlay.api.Constants;
import org.ow2.petals.topology.generated.Topology;

/**
 * Unit tests of {@link PetalsRegistryClientApi}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
// MethodSorter 'NAME_ASCENDING' required to check that the artifact registry is correctly cleared at the end of a unit
// test.
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class PetalsRegistryClientApiTest {

    /**
     * A Petals Registry Client API Mock dedicated to the test 'xxxxxRegistryClearedAtTheEnd' to check that the
     * registries are correctly cleared when leaving the unit test.
     */
    @Rule
    public final PetalsRegistryClientApi globalRegistryClientApi = new PetalsRegistryClientApi();

    @Rule
    public final PetalsRegistryClientApi registryClientApiOne = new PetalsRegistryClientApi();

    @Rule
    public final PetalsRegistryClientApi registryClientApiTwo = new PetalsRegistryClientApi();

    /**
     * <p>
     * Check that the topology registry is correctly cleared when leaving the unit test.
     * </p>
     * <p>
     * In the test {@link #topologyRegistryClearedAtTheEnd_0()}, a topology is added to the registry, the check will be
     * done in {@link #topologyRegistryClearedAtTheEnd_1()}.
     * </p>
     */
    @Test
    public void topologyRegistryClearedAtTheEnd_0() {

        this.globalRegistryClientApi.registerTopology("a-topology-id", new Topology());

        // The check is done in artifactRegistryClearedAtTheEnd_1
    }

    /**
     * See {@link #topologyRegistryClearedAtTheEnd_0()}
     */
    @Test
    public void topologyRegistryClearedAtTheEnd_1() throws ConnectionErrorException {

        assertEquals(
                "Unexpected number of topology in the topology registry",
                0,
                this.globalRegistryClientApi
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());

        // The registration of the same topology should succeed
        this.globalRegistryClientApi.registerTopology("a-topology-id", new Topology());

    }

    /**
     * Check that the topology registrations two Petals Registry Client API Mock are correctly isolated.
     */
    @Test
    public void oneTopologyRegistryPerPetalsAdminMockInstance() throws ConnectionErrorException {

        this.registryClientApiOne.registerTopology("a-topology-id", new Topology());

        assertEquals(
                1,
                this.registryClientApiOne
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());
        assertEquals(
                0,
                this.registryClientApiTwo
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());

        this.registryClientApiTwo.registerTopology("another-topology-id", new Topology());

        assertEquals(
                1,
                this.registryClientApiOne
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());
        assertEquals(
                1,
                this.registryClientApiTwo
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());
    }

    /**
     * <p>
     * Unregister a topology previously registered.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the topology is correctly unregistered</li>
     * </ul>
     * </p>
     */
    @Test
    public void unregisterTopologyPreviouslyRegistered() throws ConnectionErrorException {

        this.registryClientApiOne.registerTopology("a-topology-id", new Topology());

        assertEquals(
                1,
                this.registryClientApiOne
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());

        assertTrue(this.registryClientApiOne.unregisterTopology("a-topology-id"));
        assertEquals(
                0,
                this.registryClientApiOne
                        .getPetalsRegistryClientFactory()
                        .createClient("localhost", Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP,
                                Constants.DEFAULT_CREDENTIALS_PASSWORD).getTopologies().size());

    }

    /**
     * <p>
     * Unregister an unexisting topology.
     * </p>
     * <p>
     * Expected results: <code>false</code> is returned because the topology is not registered.
     * </p>
     */
    @Test
    public void unregisterUnexistingTopology() {

        assertFalse(this.registryClientApiOne.unregisterTopology("unknown-topology-id"));

    }
}
